
#ifndef BGCODE_CONVERT_EXPORT_H
#define BGCODE_CONVERT_EXPORT_H

#ifdef BGCODE_CONVERT_STATIC_DEFINE
#  define BGCODE_CONVERT_EXPORT
#  define BGCODE_CONVERT_NO_EXPORT
#else
#  ifndef BGCODE_CONVERT_EXPORT
#    ifdef bgcode_convert_EXPORTS
        /* We are building this library */
#      define BGCODE_CONVERT_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define BGCODE_CONVERT_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef BGCODE_CONVERT_NO_EXPORT
#    define BGCODE_CONVERT_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef BGCODE_CONVERT_DEPRECATED
#  define BGCODE_CONVERT_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef BGCODE_CONVERT_DEPRECATED_EXPORT
#  define BGCODE_CONVERT_DEPRECATED_EXPORT BGCODE_CONVERT_EXPORT BGCODE_CONVERT_DEPRECATED
#endif

#ifndef BGCODE_CONVERT_DEPRECATED_NO_EXPORT
#  define BGCODE_CONVERT_DEPRECATED_NO_EXPORT BGCODE_CONVERT_NO_EXPORT BGCODE_CONVERT_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef BGCODE_CONVERT_NO_DEPRECATED
#    define BGCODE_CONVERT_NO_DEPRECATED
#  endif
#endif

#endif /* BGCODE_CONVERT_EXPORT_H */
